const net = require('net');

// Array para almacenar los clientes conectados
let clients = [];

// Función para difundir un mensaje a todos los clientes excepto al remitente
function broadcast(message, sender) {
    clients.forEach(client => {
        if (client !== sender && !client.destroyed) {
            client.write(message + '\n'); // Agregar un salto de línea al final del mensaje
        }
    });
    process.stdout.write(message + '\n'); // Imprimir el mensaje en la consola del servidor con un salto de línea
}

// Crear un servidor TCP
const server = net.createServer(socket => {
    // Agregar el nuevo cliente al array
    clients.push(socket);

    // Imprimir en la consola cuando un cliente se conecte
    console.log(`Cliente conectado: ${socket.remoteAddress}:${socket.remotePort}`);

    // Manejar los datos recibidos del cliente
    socket.on('data', data => {
        const message = `${socket.userName || socket.remoteAddress}:${data}`.replace(/:\d+\./g, ':'); // Eliminar la dirección IP del mensaje
        broadcast(message, socket);
    });

    // Manejar el evento 'close' del cliente
    socket.on('close', () => {
        // Eliminar el cliente desconectado del array
        clients = clients.filter(client => client !== socket);
        const message = `Cliente desconectado: ${socket.remoteAddress}:${socket.remotePort}\n`;
        broadcast(message, socket);
    });

    // Manejar el evento 'error' del cliente
    socket.on('error', error => {
        console.error(`Error en el cliente ${socket.remoteAddress}:${socket.remotePort}: ${error.message}`);
    });

    // Manejar el evento 'username' del cliente
    socket.on('username', userName => {
        socket.userName = userName;
    });
});

// Manejar el evento 'error' del servidor
server.on('error', error => {
    console.error(`Error en el servidor: ${error.message}`);
});

// Escuchar en el puerto 5555
const PORT = 5555;
server.listen(PORT, () => {
    console.log(`Servidor de chat escuchando en el puerto ${PORT}`);
});


